import unittest

from src.simple_nlp_library.preprocessing import (
    single_spaces,
    lower_letters,
    non_stopword_tokens,
    semantic_tokens,
)


class TestPreprocessing(unittest.TestCase):
    def test_single_spaces(self):
        self.assertEqual(
            single_spaces("The  quick \t brown \n fox jumps"),
            "The quick brown fox jumps",
        )

    def test_lower_letters(self):
        self.assertEqual(
            lower_letters("The 2quick brown fox jumps-over!"),
            "the quick brown fox jumps over",
        )

    def test_non_stopword_tokens(self):
        self.assertEqual(
            non_stopword_tokens(["the", "quick", "brown", "fox", "jumps"]),
            ["quick", "brown", "fox", "jumps"],
        )

    def test_semantic_tokens(self):
        self.assertEqual(
            semantic_tokens("The 2 quick \t brown fox jumps, over the lazy dog!"),
            ["quick", "brown", "fox", "jumps", "lazy", "dog"],
        )



import unittest

from src.simple_nlp_library.metrics import (
    dot_product,
    frobenius_norm,
    cosine_similarity,
    inserting_distance,
    inserting_similarity,
)


class TestMetrics(unittest.TestCase):
    def test_dot_product(self):
        self.assertEqual(dot_product([1.0, 2.0, 3.0], [4.0, 5.0, 6.0]), 32)

    def test_frobenius_norm(self):
        self.assertEqual(frobenius_norm([1.0, 1.0, 1.0, 2.0, 3.0]), 4.0)

    def test_cosine_similarity(self):
        self.assertAlmostEqual(
            cosine_similarity([1.0, 1.0, 1.0], [4.0, 4.0, 4.0]), 1.0, 1
        )

    def test_inserting_distance(self):
        self.assertEqual(inserting_distance("simple", "simple"), 0)
        self.assertEqual(inserting_distance("simple", "simpler"), 1)
        self.assertEqual(inserting_distance("easy", "hard"), 4)

    def test_inserting_distance_backward(self):
        self.assertEqual(inserting_distance("simple", "simple", True), 0)
        self.assertEqual(inserting_distance("best", "the best", True), 4)
        self.assertEqual(inserting_distance("easy", "hard", True), 4)

    def test_inserting_similarity(self):
        self.assertEqual(inserting_similarity("simple", "simple"), 1.0)
        self.assertEqual(inserting_similarity("simple", "simpler"), 1.0 - 1 / 7)
        self.assertEqual(inserting_similarity("easy", "hard"), 0.0)

    def test_inserting_similarity_backward(self):
        self.assertEqual(inserting_similarity("simple", "simple", True), 1.0)
        self.assertEqual(inserting_similarity("best", "the best", True), 0.5)
        self.assertEqual(inserting_similarity("easy", "hard", True), 0.0)



from typing import List


def dot_product(a: List[float], b: List[float]) -> float:
    return sum([x1 * x2 for x1, x2 in zip(a, b)])


def frobenius_norm(a: List[float]) -> float:
    return sum([x * x for x in a]) ** 0.5


def cosine_similarity(a: List[float], b: List[float]) -> float:
    return dot_product(a, b) / (frobenius_norm(a) * frobenius_norm(b))


def inserting_distance(word1: str, word2: str, backward: bool = False) -> int:
    common_start = 0
    for i in range(min(len(word1), len(word2))):
        if (not backward and word1[i] == word2[i]) or (
            backward and word1[-i - 1] == word2[-i - 1]
        ):
            common_start += 1
        else:
            break
    return max(len(word1), len(word2)) - common_start


def inserting_similarity(word1: str, word2: str, backward: bool = False) -> float:
    return 1.0 - (
        inserting_distance(word1, word2, backward) / max(len(word1), len(word2))
    )
